/* FILE: diag.c                 Copyright (c), Symmetric Research, 2003-2005

This program checks that the PARxCH and PARGPS hardware are working ok.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pargps.h"

#include "parxch.h"
#include "srhelper.h"





// Global variables

#define ATOD_CHANNELS (PARXCH_MAX_ANALOG_CHANNELS+PARXCH_MAX_GPS_CHANNELS)
#define MAXNAME        20
#define MAXSER         20
#define NPTS           128
#define MAXPTS        (NPTS*ATOD_CHANNELS)

char       GpsDriver[MAXNAME];
char       AtodDriver[MAXNAME], AtodModelName[MAXNAME], ParPortModeName[MAXNAME];
int        SerialPort, ParallelMode, AtodModel;
int        ParXchAnalogChannels, ParXchGpsChannels;
SERIALDATA SerialValues[MAXSER];
long       AnalogValues[MAXPTS];


char *UsageMsg[] = {
        
   "USAGE: diag GpsDriver [Serial] [AtodDriver] [AtodModel] [ParPortMode]  \n",
   "                                                                       \n",
   "Where possible values for the arguments are:                           \n",
   "                                                                       \n",
   NULL
   };

char *ExampleMsg[] = {
   "                                                                      \n",
   "EXAMPLES:                                                             \n",
   "          > diag SrParGps                                             \n",
   "          > diag SrParGps378 com2 SrParXch378 PAR4CH epp              \n",
   "          > diag bpp com1                                             \n",
   
   NULL
   };


#define CMD_DONE        -1
#define CMD_GPSDRIVER    0
#define CMD_GPSSERIAL    1
#define CMD_XCHDRIVER    2
#define CMD_XCHMODEL     3
#define CMD_XCHPARPORT   4
#define CMD_MAX          5


char *OptionHeader[CMD_MAX] = {
        "  GpsDriver = ",
        "     Serial = ",
        " AtodDriver = ",
        "  AtodModel = ",
        "ParPortMode = ",
        };

struct CommandLineOption {
        int   Category;
        char *Name;
        char *ParmChar;
        int  *ParmInt;
        int   Default;
        int   Value;
        } *Opt, OptionsTable[] = {

  CMD_GPSDRIVER,  "SrParGps",    GpsDriver,  NULL,          0, 0,
  CMD_GPSDRIVER,  "SrParGps378", GpsDriver,  NULL,          0, 0,
  CMD_GPSDRIVER,  "SrParGps278", GpsDriver,  NULL,          0, 0,

  CMD_GPSSERIAL,  "com1",        NULL,       &SerialPort,   0, 1,
  CMD_GPSSERIAL,  "com2",        NULL,       &SerialPort,   1, 2,
        
  CMD_XCHDRIVER,  "SrParXch378", AtodDriver, NULL,          1, 0,
  CMD_XCHDRIVER,  "SrParXch278", AtodDriver, NULL,          0, 0,
  CMD_XCHDRIVER,  "SrParXch3BC", AtodDriver, NULL,          0, 0,

  CMD_XCHMODEL,   "DEFAULT",     NULL,       &AtodModel,    1, PARXCH_MODEL_UNKNOWN,
  CMD_XCHMODEL,   "PAR1CH",      NULL,       &AtodModel,    0, PARXCH_MODEL_PAR1CH,
  CMD_XCHMODEL,   "PAR4CH",      NULL,       &AtodModel,    0, PARXCH_MODEL_PAR4CH,
  CMD_XCHMODEL,   "PAR8CH",      NULL,       &AtodModel,    0, PARXCH_MODEL_PAR8CH,

  CMD_XCHPARPORT, "bpp",         NULL,       &ParallelMode, 0, PARXCH_PORT_MODE_BPP,
  CMD_XCHPARPORT, "epp",         NULL,       &ParallelMode, 1, PARXCH_PORT_MODE_EPP,
  CMD_XCHPARPORT, "ecp/bpp",     NULL,       &ParallelMode, 0, PARXCH_PORT_MODE_ECP_BPP,
  CMD_XCHPARPORT, "ecp/epp",     NULL,       &ParallelMode, 0, PARXCH_PORT_MODE_ECP_EPP,

  CMD_DONE,       NULL,          NULL,       NULL,          0, 0,

        };



char *NextStep[] = {
        
   "                                                                        \n",
   "Once the PARGPS and PARxCH A/D are successfully installed, you may want \n",
   "to run one of the A/D acquisition programs scope or simp with the       \n",
   "DAT output format selected and GPS enabled to acquire data with GPS     \n",
   "time stamping.                                                          \n",
   "                                                                        \n",
   "Use the sethdrtm utility in pargps\\utils on DAT files to fill in the   \n",
   "DAT header start time from the GPS mark data stored in the body of      \n",
   "the file.                                                               \n",
   "                                                                        \n",
   "Use the dat2asc utility in the parxch\\convert\\ascii directory to      \n",
   "view the A/D data and GPS info in a simple text format.                 \n",
        
    NULL
    };





// Protos

void ProcessCommandLine( int argc, char *argv[] );
void Usage( char *msg );
void PrintMsg( char *MsgArray[], char *Str1 );







int main( int argc, char *argv[] ) {

        int i, printresult, oldIntrCount, IntrCount, NmeaZeroCount, ErrorCode;
        int MarkChan, NumChan, NumMark, FirstMark, SecondMark, ThirdMark;
        unsigned int nser, nremain, npts, nextpt;
        double ActualSps;
        DEVHANDLE GpsHandle;
        DEVHANDLE AtodHandle;
        
        
        // Display the leader message.

        printf( "PARGPS diag: (c) SR 2004 : Rev {03/01/04}\n" );



        // Allow keyboard check to return if no key pressed
        
        SrKeyboardNonBlock();



        // Get port numbers from the command line

        ProcessCommandLine( argc, argv );

        printf( "\nUsing ...\n" );
        printf( "         %s %s\n",      OptionHeader[CMD_GPSDRIVER],  GpsDriver );
        printf( "         %s COM%d\n",   OptionHeader[CMD_GPSSERIAL],  SerialPort );
        printf( "         %s %s\n",      OptionHeader[CMD_XCHDRIVER],  AtodDriver );
        printf( "         %s %s (%d)\n", OptionHeader[CMD_XCHMODEL],   AtodModelName, AtodModel );
        printf( "         %s %s (%d)\n", OptionHeader[CMD_XCHPARPORT], ParPortModeName, ParallelMode );
        
        printf( "\nPress ENTER to continue: " );  SrFlushPrint();
        SrGetKeyWait();


        
        // Open the PARGPS driver.

        printf( "Opening PARGPS driver ...     " );  SrFlushPrint();
        
        GpsHandle = ParGpsOpen( GpsDriver, SerialPort, &ErrorCode );
        if (GpsHandle == BAD_DEVHANDLE ) {
                printf( "ERROR: %s\n", PARGPS_ERROR_MSG[ErrorCode] );
                exit( 1 );
                }
        printf( "Success\n" );




        // Open the A/D driver and attach to PARGPS
        
        printf( "Opening PARxCH driver ...     " );  SrFlushPrint();
        
        AtodHandle = ParXchOpen( AtodDriver,
                                 AtodModel,
                                 ParallelMode,
                                 50.0,
                                 &ActualSps,
                                 &ErrorCode
                                 );
        if ( AtodHandle == BAD_DEVHANDLE ) {
                printf( "ERROR: %s\n\n", PARXCH_ERROR_MSG[ErrorCode] );
                printf( "     Is the PARxCH driver with the selected name installed ?\n" );
                printf( "     Is the PARxCH device you are using a %s ?\n", AtodModelName );
                printf( "     Is the %s power on ?\n", AtodModelName );
                printf( "     Is the DB25 parallel port cable connected ?\n\n" );
                ParGpsClose( GpsHandle );
                exit( 1 );
                }
        printf( "Success\n" );



        printf( "Attaching PARGPS<->PARxCH ... " );  SrFlushPrint();
        
        if ( !ParXchAttachGps( AtodHandle, GpsHandle, &ErrorCode ) ) {
                printf( "ERROR: %s\n", PARXCH_ERROR_MSG[ErrorCode] );
                ParXchClose( AtodHandle );
                ParGpsClose( GpsHandle );
                exit( 1 );
                }
        printf( "Success\n" );




        // Check for PARxCH firmware support for PARGPS

        printf( "Checking %s firmware ...  ", AtodModelName );  SrFlushPrint();


        ParGpsStart( GpsHandle );
        ParXchStart( AtodHandle );

        NumChan  = ParXchAnalogChannels + ParXchGpsChannels;
        MarkChan = NumChan-1;
        nremain  = NPTS * NumChan;
        nextpt   = 0;

        while (nremain > 0) {
                
                npts = ParXchReadDataWithGpsMark( AtodHandle,
                                                 &AnalogValues[nextpt],
                                                  nremain,
                                                 &ErrorCode
                                                 );
                nremain -= npts;
                nextpt  += npts;

                if (ErrorCode != PARXCH_ERROR_NONE) {
                        printf( "ERROR: %s\n", PARXCH_ERROR_MSG[ErrorCode] );
                        break;
                        }
                
                if (SrGetKeyCheck() != EOF) {
                        printf( "ERROR: Skipping firmware check\n");
                        break;
                        }

                SrSleep( 250 );
                }

        ParXchStop( AtodHandle);
        ParGpsStop( GpsHandle );

        
        if (nremain <= 0) {

                FirstMark = SecondMark = ThirdMark = -1;
                NumMark = 0;

                for ( i = MarkChan ; i < MAXPTS ; i += NumChan ) {
                        if (AnalogValues[i]) {
                                NumMark++;
                                if (FirstMark == -1)
                                        FirstMark = ((i+1)/NumChan)-1;
                                else if (SecondMark == -1)
                                        SecondMark = ((i+1)/NumChan)-1;
                                else if (ThirdMark == -1)
                                        ThirdMark = ((i+1)/NumChan)-1;
                                }
                        }

                // NumMark should be 128 buffer / 50 sps = 2.5 therefore 2 or 3
                // For extra tolerance, we'll allow for a 4th mark since there
                // can be a long delay of 12 or more cycles as the A/D does
                // its start up calibrations.  This also explains why the first
                // mark might not be the expected 50 samples earlier than the
                // second.  But 2-4 marks is a long way from the 64 one would
                // get with a PAR4CH having Rev 2.5 firmware which gives a
                // fake PPS signal every other sample.
                
                if ( (NumMark <= 1) || (NumMark >= 5) ) {
                        printf( "ERROR\n\n" );
                        if (NumMark <= 1) {
                                printf( "     Firmware probably OK, but no PPS marks were received.\n");
                                printf( "     Is PARGPS power on and is the red LED blinking ?\n");
                                printf( "     Is DB15 cable connected from PARGPS to %s Digital I/O ?\n\n",AtodModelName);
                                }
                        else
                                printf( "     %s Firmware 3.0 or greater is required\n\n", AtodModelName);


                        ParXchReleaseGps( AtodHandle );
                        ParXchClose( AtodHandle );
                        ParGpsClose( GpsHandle );
                        exit( 1 );
                        }
                else
                        printf( "Success\n" );
                }






        // Check for a PPS signal

        printf( "Checking PPS signal ...       " );  SrFlushPrint();
        
        ParGpsReset( GpsHandle );
        ParGpsSetInterruptMode( GpsHandle, INTERRUPT_PPS_ONLY );
        ParXchInterruptEnable( AtodHandle );
        oldIntrCount = 0;
        IntrCount    = 0;
        printresult  = 1;
        
        while (1) {
                ParGpsGetInterruptCount( GpsHandle, &IntrCount );
                if (IntrCount != oldIntrCount) {
                        if (printresult == 1) {
                                printf( "Success\n" );
                                printresult = 0;
                                }
                        printf( "   Found %d PPS\n", IntrCount);
                        oldIntrCount = IntrCount;
                        if (IntrCount > 2) {
                                printf( "   PPS good\n");
                                break;
                                }
                        }

                if (SrGetKeyCheck() != EOF) {
                        if (printresult == 1)
                                printf( "WARNING: Skipping PPS check\n");
                        break;
                        }

                SrSleep( 250 );
                }
        
        ParXchInterruptDisable( AtodHandle );



        // Check for a serial signal

        printf( "Checking NMEA strings.  Press ENTER to stop.\n" );
        
        ParGpsReset( GpsHandle );
        ParGpsSetInterruptMode( GpsHandle, INTERRUPT_ALTERNATING );
        ParXchStart( AtodHandle );
        ParGpsStart( GpsHandle );

        printresult   = 1;
        NmeaZeroCount = 0;
        
        while (1) {
                
                nser = ParGpsReadSerialData( GpsHandle,
                                             SerialValues,
                                             MAXSER,
                                             &ErrorCode
                                           );

                if (ErrorCode != PARGPS_ERROR_NONE) {
                        printf( "ERROR: %s\n", PARGPS_ERROR_MSG[ErrorCode] );
                        break;
                        }
                        
                if (nser > 0) {
                        if (printresult == 1)
                                printresult = 0;
                        if (SerialValues[0].NmeaCount > 0) {
                                printf( "\n   PPS Mark # %d, NMEA String Count %d\n",
                                          SerialValues[0].PpsEventNum,
                                          SerialValues[0].NmeaCount );
                                for ( i = 0 ; i < SerialValues[0].NmeaCount ; i++ ) {
                                        printf( "   %d: %s",
                                                  i, &SerialValues[0].NmeaMsg[i*MAX_NMEA_SIZE]);
                                        }
                                } // end if NmeaCount > 0

                        else { // (SerialValues[0].NmeaCount == 0)
                                printf( "\n   PPS Mark # %d, NMEA No Strings Received\n",
                                          SerialValues[0].PpsEventNum,
                                          SerialValues[0].NmeaCount);
                                NmeaZeroCount++;
                                if (NmeaZeroCount > 2) {
                                        printf( "\nERROR: No NMEA strings are being received\n" );
                                        printf( "Please check serial cable.\n" );
                                        ParGpsStop( GpsHandle );
                                        ParXchInterruptDisable( AtodHandle );
                                        ParXchReleaseGps( AtodHandle );
                                        ParXchClose( AtodHandle );
                                        ParGpsClose( GpsHandle );
                                        exit( 1 );
                                        }
                                } // end else NmeaCount == 0

                        } // end if nser > 0

                
                if (SrGetKeyCheck() != EOF) {
                        if (printresult == 1)
                                printf( "WARNING: Skipping serial check\n");
                        break;
                        }

                SrSleep( 250 );
                }
        
        ParGpsStop( GpsHandle );
        ParXchStop( AtodHandle );


        
        // Close drivers
        
        ParXchReleaseGps( AtodHandle );
        ParXchClose( AtodHandle );
        ParGpsClose( GpsHandle );



        PrintMsg( NextStep, "" );

        
        exit( 0 );
}



void ProcessCommandLine( int argc, char *argv[] ) {

        int i, CmdFilled[CMD_MAX];

        // Set defaults

        SerialPort   = 2;
        ParallelMode = PARXCH_PORT_MODE_EPP;
        AtodModel    = PARXCH_MODEL_UNKNOWN;
        SrStringCopy( GpsDriver, "SrParGps", MAXNAME );
        SrStringCopy( AtodDriver, "SrParXch378", MAXNAME );

        for ( i = 0 ; i < CMD_MAX ; i++ )
                CmdFilled[i] = 0;
        

        // Get the command line arguments.

        if ( argc < 2 || argc > 6 )
                Usage( "Wrong number of arguments.  Must be 1 to 5." );


        while ( --argc ) {

                argv++;
                Opt = OptionsTable;

                
                // Check all names in the OptionsTable for a match.
                
                while ( Opt->Name && strcmp( *argv, Opt->Name ) )
                                Opt++;


                // Argument did not match a valid option.

                if ( !Opt->Name )
                        Usage( "Unknown option" );



                // Check for duplicate argument.

                if ( CmdFilled[ Opt->Category ] == 1 )
                        Usage( "Bad command line with more than one option per category" );
                else
                         CmdFilled[ Opt->Category ] = 1;

                
                // Fill in known option.

                if ( Opt->ParmInt )
                        *Opt->ParmInt = Opt->Value;
                
                else if ( Opt->ParmChar )
                        SrStringCopy( Opt->ParmChar, Opt->Name, MAXNAME );

                else
                        Usage( "Option of unknown type" );


                } // end while argc



        // Fill mode name
        
        switch (ParallelMode) {

                case PARXCH_PORT_MODE_BPP:
                        SrStringCopy( ParPortModeName, "BPP", MAXNAME );
                        break;

                case PARXCH_PORT_MODE_EPP:
                        SrStringCopy( ParPortModeName, "EPP", MAXNAME );
                        break;

                case PARXCH_PORT_MODE_ECP_BPP:
                        SrStringCopy( ParPortModeName, "ECP/BPP", MAXNAME );
                        break;

                case PARXCH_PORT_MODE_ECP_EPP:
                        SrStringCopy( ParPortModeName, "ECP/EPP", MAXNAME );
                        break;

                default:
                        SrStringCopy( ParPortModeName, "UNKNOWN", MAXNAME );
                        break;
                }
        
        
        // Fill PARxCH device dependent parameters

        if (AtodModel == PARXCH_MODEL_UNKNOWN)
                AtodModel = ParXchGetDefaultModel( AtodDriver );
        
        switch (AtodModel) {

                case PARXCH_MODEL_PAR1CH:
                        ParXchAnalogChannels = PAR1CH_ANALOG_CHANNELS;
                        ParXchGpsChannels    = PAR1CH_GPS_CHANNELS;
                        SrStringCopy( AtodModelName, PAR1CH_MODELNAME, MAXNAME );
                        break;

                case PARXCH_MODEL_PAR4CH:
                        ParXchAnalogChannels = PAR4CH_ANALOG_CHANNELS;
                        ParXchGpsChannels    = PAR4CH_GPS_CHANNELS;
                        SrStringCopy( AtodModelName, PAR4CH_MODELNAME, MAXNAME );
                        break;

                case PARXCH_MODEL_PAR8CH:
                        ParXchAnalogChannels = PAR8CH_ANALOG_CHANNELS;
                        ParXchGpsChannels    = PAR8CH_GPS_CHANNELS;
                        SrStringCopy( AtodModelName, PAR8CH_MODELNAME, MAXNAME );
                        break;

                default:
                        Usage( "PARxCH model unknown" );
                        break;
                }
        
        
        return;
}

void Usage( char *msg ) {

        int First, PrevCat, Ndefault;

        // Print optional error string

        if (msg)
                printf( "\nERROR: %s\n", msg);


        // Print a command line usage message and quit.

        PrintMsg( UsageMsg, "" );

        First    = 1;
        Ndefault = 0;
        Opt      = OptionsTable;
        PrevCat  = Opt->Category;

        while ( Opt->Name ) {
                if (First) {
                        printf( "    %s", OptionHeader[Opt->Category] );
                        First = 0;
                        }
                printf( "%s", Opt->Name );

                if (Opt->Default)
                        Ndefault++;

                Opt++;  // Increment to next option
                
                if ( Opt->Category == PrevCat )
                        printf( ", " );
                else {
                        First = 1;
                        PrevCat = Opt->Category;
                        printf( "\n" );
                        }
                }


        printf( "\n    Defaults are: " );
        
        Opt = OptionsTable;

        while ( Opt->Name ) {
                
                if (Opt->Default) {
                        printf( "%s", Opt->Name );
                        if (Ndefault > 1) {
                                printf( ", " );
                                Ndefault--;
                                }
                        }

                Opt++;     // Increment to next option
                }
        printf( "\n" );

        PrintMsg( ExampleMsg, "" );

        exit( 1 );
}



void PrintMsg( char *MsgArray[], char *Str1 ) {

        int i;

        // Print the message array.

        for ( i = 0 ; MsgArray[i] ; i++ )

                printf( MsgArray[i], Str1, Str1 );
}
