/* FILE: srhelper.h               Copyright (c), Symmetric Research, 2002

SR helper functions C user library include file.

These helper functions a used to shield the rest of the code from
variations between different OS's.

*/



/* FUNCTION TYPE DEFINES:

Most of the SR software links in a static copy of the PAR4CH library.
For static linking, function and global variable declarations are
easy and standard across operating systems.  The following defines
default to static linking if not otherwise specified.

Supporting Windows dynamic DLL libraries introduces complications.
The following ifdef defines allow for one PAR4CH library source to
service either statically linked or DLL applications.

Linux shared object libraries are also supported but do not require any
compile time specificatons beyond static.

*/

#if defined( _WIN_DLLEXPORT )
#define FUNCTYPE( type )        __declspec( dllexport ) type __stdcall
#define VARTYPE( type )         __declspec( dllexport ) type

#elif defined( _WIN_DLLIMPORT )
#define FUNCTYPE( type )        __declspec( dllimport ) type __stdcall
#define VARTYPE( type )         __declspec( dllimport ) type

#else           // default to simple declarations for static linking
#define FUNCTYPE( type )        type
#define VARTYPE( type )         type

#endif



#if defined( SROS_LINUX )
#include <unistd.h> // for STDOUT_FILENO
#define FH_STDOUT   STDOUT_FILENO
#define O_BINARY    0x00
#define SR_DIRSLASH '/'
#define SR_QENTER "Q+Enter"
#else
#define FH_STDOUT   _fileno( stdout )
#define SR_DIRSLASH '\\'
#define SR_QENTER "Q"
#endif



/* GENERIC HELPER FUNCTIONS:

These functions perform a few generic keyboard and file IO tasks and
are used in various SR application programs to help smooth over OS
differences.

*/

FUNCTYPE( int )  SrHelperRev( int* Rev );
FUNCTYPE( int  ) SrKeyboardNonBlock( void );
FUNCTYPE( int  ) SrGetKeyCheck( void );
FUNCTYPE( int  ) SrGetKeyWait(  void );
FUNCTYPE( void ) SrFlushPrint( void );
FUNCTYPE( int  ) SrSleep( int ms );
FUNCTYPE( int  ) SrWait( int ms );
FUNCTYPE( int  ) SrFileLength( int fh );
FUNCTYPE( int  ) SrFileExist( char *filename );
FUNCTYPE( int  ) SrCommit( int fh );
FUNCTYPE( int  ) SrStringCopy( char *DestStr, const char *SrcStr, int MaxChar );
