// FILE: view.h               Copyright (c), Symmetric Research, 2003-2004
//
// Header file for generic portion of the View program includes
//
//      CViewApp       main app    controls data, files, and registry
//      CViewWindow    main window controls menus and graphics
//      CViewDialog    base dialog provides validation + etc functions
//      CDisplayDlg    dialog for display options
//      CGotoPtDlg     dialog for getting selected point
//      CContentsDlg   dialog for help contents
//      CTopicDlg      dialog for help topics
//



//***********************************************************************


// WINDOWS CONSTANTS

#include <afxwin.h>
#include <afxext.h>     // needed for status bar


// RESOURCE CONSTANTS

#include "view.rch"


// SRDAT HEADER STRUCTURE AND CONSTANTS
// SR HELPER FUNCTIONS
// PARXCH SPECIFIC FUNCTIONS

extern "C" {
#include "srdat.h"
#include "srhelper.h"
#include "parxch.h"
}




// APPLICATION CONSTANTS


#define WM_EDITLOSTFOCUS        (WM_USER+100)

#define UNCHECKED               0
#define CHECKED                 1
#define INDETERMINATE           2

#define MASK_CHECK              0X03L
#define MASK_HIGHLIGHT          0X04L
#define MASK_FOCUS              0X08L

#define GUTTER                  10
#define HSTARTBAR               25
#define HMARGIN                 10
#define VMARGIN                 5
#define HTITLEMARGIN            128

#define CS_MONO                 0
#define CS_GREEN                1
#define CS_BLUE                 2
#define CS_BROWN                3

#define FILEFMT_NONE            0
#define FILEFMT_OUT             1
#define FILEFMT_DAT             2

#define CH_TYPE_NONE            0
#define CH_TYPE_ANALOG          1
#define CH_TYPE_DIGITAL         2
#define CH_TYPE_GPSMARK         3
#define CH_TYPE_GPSCOUNTER      4

#define CONVERT_OFFSET          0x00000000
#define CONVERT_SIGNED          0x00800000

#define MAXTITLE                12
#define MAXSTR                  256

#define DATA_PT_SIZE            sizeof(long)

#define DLG_POS_INIT            -999


// This is defined to be the maximum for all A/D's that
// will work with view.
//
//    A/D        Max Channels (A+D+G+C)
//   PAR8CH          8+1+1+1=11
//   PAR4CH          4+0+1+0=5
//   PAR1CH          1+0+1+0=2 (5)
//

#define MAXHDRCHANNEL           11




struct channel_data {
        long  channel;
        char  title[MAXTITLE];
        };


typedef struct HDR_DATA {
        char   Id[32];
        long   Channels;
        long   RecordPts;
        long   WordSize;
        long   DataCoding;
        double SampleRate;
        double Gain;
        double FilterCoeff;
        double FilterScale;
        struct channel_data ChannelList[MAXHDRCHANNEL];
        } SROUT_HDR;



// Needed for forward reference
class CViewWindow;




//***********************************************************************


class CViewApp : public CWinApp {

public:

        // Data

        int    DefaultScreenWidth;
        int    DefaultScreenHeight;
        int    DefaultNsamples;
        int    MinAnalogChannel;
        int    MaxAnalogChannel;
        int    MinDigitalChannel;
        int    MaxDigitalChannel;
        int    MinGpsChannel;
        int    MaxGpsChannel;
        int    MinCounterChannel;
        int    MaxCounterChannel;
        int    MinTotalChannel;
        int    MaxTotalChannel;
        int    MinAnaPlotChannel;
        int    MaxAnaPlotChannel;
        int    MinDigPlotChannel;
        int    MaxDigPlotChannel;
        int    MinGpsPlotChannel;
        int    MaxGpsPlotChannel;
        int    MinCntPlotChannel;
        int    MaxCntPlotChannel;
        int    MinTotalPlotChannel;
        int    MaxTotalPlotChannel;
        int    MaxSample;
        int    MinAmpShift;
        int    MaxAmpShift;

        int    Nchannels;
        int    NsamplesBuff;
        int    NsamplesPlot;
        int    NchannelsPlot;
        int    Nplotbuffers;

        long   Nbufferpts;
        long   NbufferptsPlot;
        long   Nbufferbytes;
        long   NbufferbytesPlot;
        long   LastNbufferbytes;
        long   LastNbufferbytesPlot;

        double *pTMPoffset;
        long   *pDCoffset;
        int    *pPlotChType;
        char   *pPlotTitle;
        long   *pDemuxArray;
        
        BOOL   IsAllocated;
        BOOL   ResetPaint;
        BOOL   ResetDC;

        int    ErrCode;
        HWND   hWndPrev;

        CString CurrDir;
        CString OpenDir;
        CString AppTitle;
        CString AppRev;
        CString AppClassName;
        CString ErrString;

        int     GpsMarkChannel;

        char szTemp[MAXSTR];
        char SrDatHdrPaddingMFC[SRDAT_HDR_PADDINGSIZE];

        SROUT_HDR OutHdr;
        SRDAT_HDR DatHdr;

        long    ConvertToOffset;
        long   *pDataArray;
        
        BOOL    DoStatusBar;

        float   AcqSPS;

        int     PltShiftBy;
        int     PltNdisplay;
        int     PltFirstSample;
        int     PltFirstTrace;
        int     PltColorScheme;
        BOOL    PltClipping;
        BOOL    PltAutoDC;

        long   *pScanActive;
        int    *pScanChType;
        int    *pScanOrder;
        char   *pScanTitle;
        char   *pDigTitle;

        BOOL    DigitalEnable;
        BOOL    GpsEnable;
        BOOL    CounterEnable;
        
        CString HdrFullName;
        CString HdrName;
        CString DataFullName;
        CString DataPath;
        CString DataName;
        CString DataPrefix;
        CString DataExt;
        CString DataCurrName;
        int     DataFormat;
        int     DataDoSeq;
        int     DataSeqHome;
        int     DataPtsPerFile;
        int     DataPtsPerBuff;
        int     DataBuffPerFile;
        int     DataBytesPerPt;
        int     DataCurrentPt;
        int     DataMinPt;
        int     DataMaxPt;

        int     XchModel;
        


        // Methods

        //{{AFX_VIRTUAL(CViewApp)
        virtual BOOL InitInstance();
        virtual int  ExitInstance();
        //}}AFX_VIRTUAL


        // Other functions

        CViewWindow* NewViewWindow();
        int  ProcessCommandLine( void );
        int  SetFileInfo( int DataFmt, CString &Name1, CString &Name2 );
        int  IsFilePrefixNumeric( const char *FilePrefix, int *Number );
        int  IsFileReadable( const char *FileName );
        int  InitAppData( void );
        int  ClearChannels( void );
        int  ComputeNchannelsPlot( void );
        int  AllocateDataArray( void );
        int  ComputeBufferSizes( void );
        int  AllocateBuffers( void );
        int  UpdatePlotTraceInfo( void );
        int  ReadHdrFromFile( void );
        int  ComputeBoundingPts( void );
        int  GetFilePosForPt( int Pt, int *FileSeq, int *FileRec, int *FilePt );
        int  ReadDataFromFile( int FileSeq, int FileRec, int FilePt, int NumPts, int FirstPt );
        int  DemuxData( void );
        int  SrDatHdrReadMFC( CFile *TmpFile, SRDAT_HDR *Hdr );
        int  SrDatTagReadMFC( CFile *TmpFile, long *DataType, unsigned int *Nbytes );
        int  SrDatReadMFC( CFile *TmpFile, void *Buffer, unsigned int Nbytes );
        int  SrDatSkipMFC( CFile *TmpFile, unsigned int Nbytes );
        int  ShowHelp( int titleID, int textID, CWnd *caller );
        int  LoadHelpTitles( CListBox *listbox );
        int  AddResourceString( CListBox *listbox, int titleID, int textID, int pos = -1 );
        void AppendFileExceptionCause( CFileException *e );
        int  ShowMsg( const char *Message, const char *Title );
        void ShowError( void );
        void ExitError( void );


        //{{AFX_MSG(CViewApp)
        //}}AFX_MSG


        DECLARE_MESSAGE_MAP()
};






//***********************************************************************

class CViewWindow : public CFrameWnd {


public:

        // Data

        DWORD ColorBack;
        DWORD ColorAxis;
        DWORD ColorTitle;
        DWORD ColorTrace;

        CPen* PenBack;
        CPen* PenAxis;
        CPen* PenTitle;
        CPen* PenTrace;

        LOGFONT font;
        CFont*  CurrFont;

        int minPoint;
        int midPoint;
        int maxPoint;
        int cxChar;
        int cyChar;
        int MaxTitleLength;

        int DisplayPosTop;
        int DisplayPosLeft;
        int ContentsPosTop;
        int ContentsPosLeft;
        int TopicPosTop;
        int TopicPosLeft;
        int TopicPosBottom;
        int TopicPosRight;

        int TraceStart;
        int TraceEnd;
        int TraceWidth;
        int TraceHeight;

        int *pYlast;
        int *pYaxis;
        int *pYclipmin;
        int *pYclipmax;

        long LastPtsPlot;

        POINT *pPtArray;
        POINT *pOldArray;


protected:
        CStatusBar m_wndStatusBar;
        CScrollBar m_wndVScroll;

        int  controlkey;
        BOOL IsDragging;
        BOOL PaintDeferred;



        // Methods
public:

        //{{AFX_VIRTUAL(CViewWindow)
        virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
        virtual void PostNcDestroy();
        //}}AFX_VIRTUAL


        CViewWindow();
        virtual ~CViewWindow();

        void DrawBackground( CDC *pDC );
        void DrawTitles(     CDC *pDC );
        void DrawAxes(       CDC *pDC );
        void DrawTraces(     CDC *pDC );
        void FillTraces( int npoints );
        void UpdateStatusBar();
        int  UpdateData();
        void UpdateTitleTraces();
        void UpdateColorScheme();
        void SetColors();
        void CreatePens();
        void DeletePens();
        void CreateFont();
        void DeleteFont();
        void DeletePlotArrays();
        void DeleteVerticalBuffers();
        void UpdateTextSize( CDC *pDC );
        void SetClipLimits( int height, int traceHalf );
        void SetWindowTitle();
        void SetPaneString();
        void SetScrollBar( int nBar );
        void GetSize( int *height, int *width );
        int  AllocatePlotBuffers( );
        int  ChangeFontSize( int newsize );
        int  ComputeMaxTitleLength( CDC *pDC );
        int  ComputePlotPositions( );
        void InitDialogPositions( );
        void GetDialogPosition(  RECT *r, int nID );
        void SaveDialogPosition( RECT *r, int nID );




        //{{AFX_MSG(CViewWindow)
        virtual int  OnCreate(LPCREATESTRUCT lpCreateStruct);
        afx_msg void OnSize(UINT nType, int cx, int cy);
        afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
        afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
        afx_msg void OnPaint();
        afx_msg void OnDestroy();
        afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
        afx_msg void OnKeyUp(  UINT nChar, UINT nRepCnt, UINT nFlags);
        afx_msg void OnClose();
        afx_msg void OnSetClip();
        afx_msg void OnSetDC();
        afx_msg void OnSetAutoDC();
        afx_msg void OnShowNextBuffer();
        afx_msg void OnShowPrevBuffer();
        afx_msg void OnShowGotoBuffer();
        afx_msg void OnSetStatusBar();
        afx_msg void OnOpenFileDialog();
        afx_msg void OnDisplayDialog();
        afx_msg void OnContentsDialog();
        afx_msg void OnAboutDialog();
        afx_msg void OnDebug();
        afx_msg void OnUpdateClipCheck(   CCmdUI* pCmdUI);
        afx_msg void OnUpdateAutoDCCheck( CCmdUI* pCmdUI);
        //}}AFX_MSG

        afx_msg LRESULT OnEnterSizeMove( WPARAM wParam, LPARAM lParam);
        afx_msg LRESULT OnExitSizeMove(  WPARAM wParam, LPARAM lParam);

        DECLARE_MESSAGE_MAP()
};





//***********************************************************************


class CViewDialog : public CDialog {


public:
        CViewDialog(UINT nIdTemplate, CWnd* pParent = NULL);


        // Data

        int  DialogId;
        int  CancelId;
        BOOL DoValidate;

protected:

        int   CheckFocus( void );
        int   ValidateAsInt(   int nID, int minval, int maxval, int *theval, char *txt );
        int   ValidateAsLong(  int nID, long minval, long maxval, long *theval, char *txt );
        int   ValidateAsHex(   int nID, int minval, int maxval, int *theval, char *txt );
        int   ValidateAsFloat( int nID, float minval, float maxval, float *theval, char *txt );
        int   SetDlgItemHex(   int nID, int   val );
        int   SetDlgItemFloat( int nID, float val );
        float GetDlgItemFloat( int nID, BOOL *lpTrans = NULL);
        BOOL  SetTextReadOnly( int nID );
        void  RestorePosition();
        void  RememberPosition();


        // Generated message map functions

        //{{AFX_MSG(CViewDialog)
        afx_msg void OnActivateApp( BOOL bActive, HTASK hTask );
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        virtual BOOL OnInitDialog( int OkButtonId );
        virtual void OnCancel();
        virtual void OnOK();
        //}}AFX_MSG

        DECLARE_MESSAGE_MAP()
};



//***********************************************************************

class CAboutDlg : public CDialog {

public:
        CAboutDlg(CWnd* pParent = NULL);   // standard constructor

        //{{AFX_DATA(CAboutDlg)
        enum { IDD = IDD_ABOUTBOX };
        //}}AFX_DATA


protected:

        // Generated message map functions

        //{{AFX_MSG(CAboutDlg)
        virtual BOOL OnInitDialog();
        virtual void OnOK();
        //}}AFX_MSG

        DECLARE_MESSAGE_MAP()
};




//***********************************************************************

class CMsgDlg : public CDialog {

public:
        CMsgDlg(CWnd* pParent = NULL);         // standard
        CMsgDlg(const char *Message,           // new
                const char *Title,
                CWnd* pParent = NULL);       

        //{{AFX_DATA(CMsgDlg)
        enum { IDD = IDD_MSGBOX };
        //}}AFX_DATA

        CString m_MsgText;
        CString m_TitleText;

protected:

        // Generated message map functions

        //{{AFX_MSG(CMsgDlg)
        virtual BOOL OnInitDialog();
        virtual void OnOK();
        //}}AFX_MSG

        DECLARE_MESSAGE_MAP()
};




//***********************************************************************

class CDisplayDlg : public CViewDialog {


public:
        CDisplayDlg(CWnd* pParent = NULL);   // standard constructor


        // Dialog Data

        //{{AFX_DATA(CDisplayDlg)
        enum { IDD = IDD_DISPLAY };
        CEdit   m_TXT_ShiftBy;
        CEdit   m_TXT_Ndisplay;
        CEdit   m_TXT_FirstSample;
        CEdit   m_TXT_FirstTrace;
        CStatic m_LAB_PlotBits;
        CStatic m_LAB_Nscan;
        CStatic m_LAB_LastSample;
        CButton m_CHK_Clipping;
        CButton m_CHK_AutoDC;
        CButton m_OPT_ColorMono;
        CButton m_OPT_ColorGreen;
        CButton m_OPT_ColorBrown;
        CButton m_OPT_ColorBlue;
        int     m_ShiftBy;
        int     m_Ndisplay;
        int     m_FirstSample;
        int     m_FirstTrace;
        int     m_ColorScheme;
        int     m_TraceWidth;
        BOOL    m_Clipping;
        BOOL    m_AutoDC;
        //}}AFX_DATA



        // ClassWizard generated virtual function overrides
        //{{AFX_VIRTUAL(CDisplayDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


protected:

        void SetPlotBitsLabel( void );
        void SetNscanLabel( void );
        void SetLastSampleLabel( void );
        void SetColorSchemeRadio( void );


        // Generated message map functions

        //{{AFX_MSG(CDisplayDlg)
        virtual BOOL OnInitDialog();
        virtual void OnCancel();
        virtual void OnOK();
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        afx_msg void OnHelp();
        afx_msg void OnKillfocusTxtShiftby();
        afx_msg void OnKillfocusTxtNdisplay();
        afx_msg void OnKillfocusTxtFirstsample();
        afx_msg void OnKillfocusTxtFirsttrace();
        afx_msg void OnChkClipping( );
        afx_msg void OnChkAutoDC( );
        afx_msg void OnOptColorScheme( );
        //}}AFX_MSG
        DECLARE_MESSAGE_MAP()


};





class CGotoPtDlg : public CViewDialog {


public:
        CGotoPtDlg(CWnd* pParent = NULL);   // standard constructor


        // Dialog Data

        //{{AFX_DATA(CGotoPtDlg)
        enum { IDD = IDD_GOTOPOINT };
        CEdit   m_TXT_NewPt;
        int     m_NewPt;
        //}}AFX_DATA



        // ClassWizard generated virtual function overrides
        //{{AFX_VIRTUAL(CGotoPtDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


protected:

        void CorrectToBufferEdge( void );
        
        // Generated message map functions

        //{{AFX_MSG(CGotoPtDlg)
        virtual BOOL OnInitDialog();
        virtual void OnCancel();
        virtual void OnOK();
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        afx_msg void OnHelp();
        afx_msg void OnKillfocusPoint();
        //}}AFX_MSG
        DECLARE_MESSAGE_MAP()


};


//***********************************************************************


class CContentsDlg : public CViewDialog {

public:
        CContentsDlg(CWnd* pParent = NULL);   // standard constructor

        // Dialog Data

        //{{AFX_DATA(CContentsDlg)
        enum { IDD = IDD_HELPCONTENTS };
        CListBox        m_LST_topics;
        //}}AFX_DATA


        // ClassWizard generated virtual function overrides
        //{{AFX_VIRTUAL(CContentsDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


protected:

        // Generated message map functions

        //{{AFX_MSG(CContentsDlg)
        afx_msg void OnBtnDisplay();
        afx_msg void OnHelp();
        virtual void OnCancel();
        virtual BOOL OnInitDialog();
        afx_msg void OnDblclkTopics();
        //}}AFX_MSG

        DECLARE_MESSAGE_MAP()
};






//***********************************************************************

class CTopicDlg : public CViewDialog {

public:

        // Data

        CString htitle;
        CString htext;
        int     m_TitleID;
        int     m_TextID;

        //{{AFX_DATA(CTopicDlg)
        enum { IDD = IDD_HELPTOPIC };
        CEdit   m_TXT_Topic;
        //}}AFX_DATA



        // Methods

        CTopicDlg(CWnd* pParent = NULL);                          // standard
        CTopicDlg(int TitleID, int TextID, CWnd* pParent = NULL); // new


        // ClassWizard generated virtual function overrides

        //{{AFX_VIRTUAL(CTopicDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


protected:

        // Generated message map functions

        //{{AFX_MSG(CTopicDlg)
        virtual BOOL OnInitDialog();
        afx_msg void OnSize(UINT nType, int cx, int cy);
        afx_msg void OnSetfocusTopic();
        //}}AFX_MSG

        DECLARE_MESSAGE_MAP()
};
